----------------------------------------------------------------------------
-- project:			
-- file: 			main.lua
-- author:			Michael Groll
-- version:			
-- gitHub:			
--
-- description:    
----------------------------------------------------------------------------
-- Tables of game data
states = {} -- a table to hold my states
paddles = {} -- a table to hold my paddles objects
balls = {} -- a table to hold my balls
deleted = { order = math.huge }
-- Other variables
WIDTH = love.graphics.getWidth()
HEIGHT = love.graphics.getHeight()
bigFont = love.graphics.newFont('art/quadrangle.otf', 45)
mediumFont = love.graphics.newFont('art/quadrangle.otf', 30)
smallFont = love.graphics.newFont('art/quadrangle.otf', 12)


-- Libraries
require 'src/middleclass'
vector = require 'src/vector' -- part of the HUMP library
-- Custom classes
require 'src/button'
require 'src/paddle'
require 'src/ball'
-- States
require 'src/title'
require 'src/game'
require 'src/pregame'
require 'src/postgame'
require 'src/pause'
require 'src/help'
require 'src/options'



function love.load()
	-- Physics
	love.physics.setMeter(30) -- the number of pixels a meter will be
	world = love.physics.newWorld(0, 0, true) -- x gravity, y gravity, allowed to sleep
		world:setCallbacks(beginContact, endContact, preSolve, postSolve)
	-- Side Walls
	leftSide = {}
	leftSide.body = love.physics.newBody(world, 1, HEIGHT/2, "static")
	leftSide.shape = love.physics.newRectangleShape(3, HEIGHT)
	leftSide.fixture = love.physics.newFixture(leftSide.body, leftSide.shape)
	leftSide.fixture:setUserData("Wall")
	rightSide = {}
	rightSide.body = love.physics.newBody(world, WIDTH-1, HEIGHT/2, "static")
	rightSide.shape = love.physics.newRectangleShape(3, HEIGHT)
	rightSide.fixture = love.physics.newFixture(rightSide.body, rightSide.shape)
	rightSide.fixture:setUserData("Wall")	
	-- create our states
	game = GameState()
	pregame = PreGameState()
	postgame = PostGameState()
	help = HelpState()
	options = OptionState()
	title = TitleState() -- same as calling TitleState:new(), this also calls and sends arguments to initialize.
	pushState(title) -- start in the title state
end

function love.draw()
	for i, state in ipairs(states) do  --here we set the value inside the states table to be state
		if i == #states then    --here we only want to draw the state on the top of the stack, which is the last in the table.
			state:draw()   --by making state the value, it makes our value:function call look nicer, and we remember what we are doing!
		else
			if state.alwaysDraw  then state:draw() end
		end
	end
end

function love.update(dt)
	for i, state in ipairs(states) do
		if i == #states then
			state:update(dt)
		else
			if state.alwaysUpdate then state:update(dt) end
		end
	end
end

------------- State Functions ------------------------------------------------------

function pushState(state)
	table.insert(states,state) --puts state into the table at #state, aka, the back, or in our case on top of the stack
	state:setCallbacks()
end

function switchState(state)	--switches with the state on top of the stack
	popState() --kills the previous state
	pushState(state) 
end

function popState()
	states[#states]:destroy()
	table.remove(states,#states)
	
	if next(states) then
		states[#states]:setCallbacks()
	end
end

--------------- Custom Sort Function -----------------------------------------------
function customSort(t) --takes a table, must have a .order variable
	-- sort by the order, deleted will always be math.huge and be at the end!
	table.sort(t, function(a,b) return a.order < b.order end)
	local last = #t
	while t[last] == deleted do
		t[last] = nil
		last = last - 1
	end
end
-------------- Physics Callbacks ------------------------------------------------

function beginContact(a, b, coll)
	local n = vector.new(coll:getNormal()) --our normal vector
	for _, ball in ipairs(balls) do
		if ball.fixture == a or ball.fixture == b then
			v = vector.new(ball.body:getLinearVelocity())
			v_ = v:mirrorOn(n)
			ball.hv, ball.vv = v_:unpack()
		end
	end
end

function endContact(a, b, coll)
end

function preSolve(a, b, coll)

end

function postSolve(a, b, coll)
end